;;; pointdir.el --- various directory at point things

;; Copyright 2010, 2011, 2012, 2013, 2014, 2019 Kevin Ryde

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0 ...
;; Keywords: convenience, completion
;; URL: http://user42.tuxfamily.org/pointdir/index.html

;; pointdir.el is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; pointdir.el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.

;;; Commentary:
;; 

;;; Emacsen:
;;
;; Designed for Emacs 21 and up, works in XEmacs 21.

;;; Install:
;;
;; Put pointdir.el in one of your `load-path' directories, and to make
;; `M-x pointdir-mode' available add to your .emacs
;;
;;     (autoload 'pointdir-mode "pointdir" nil t)
;;
;; To enable from .emacs the suggestion is the following, to load and enable
;; the first time you use the minibuffer
;;
;;     (autoload 'pointdir-enable-setup "pointdir")
;;     (add-hook 'minibuffer-setup-hook 'pointdir-enable-setup)
;;
;; There's autoload cookies for the functions if you know how to use
;; `update-file-autoloads' and friends, then just `M-x' or `add-hook' as
;; desired.

;;; History:
;; 
;; Version 1 - the first version

;;; Code:

(defun pointdir-directory ()
  "Return current at point, paying attention to \"cd\" commands."
  (save-restriction
    (narrow-to-region (field-beginning) (point))
    (let (lst)
      (save-excursion
        (while (and (re-search-backward
                     "\\(?:^\\|;\\)[ \t]*cd ['\"]?\\([^'\";\r\n]+\\)"
                                        nil t)
                    (if (= (match-end 0) (point-max))
                        t ;; point is in a cd command, keep looking
                      (not (file-name-absolute-p
                            (car
                             (push (substitute-in-file-name (match-string 1))
                                   lst))))))))
      (let ((ret (pop lst)))
        (while lst
          (setq ret (expand-file-name (pop lst) ret)))
        (or ret default-directory)))))
;; cd $HOME
;; cd ..
;; cd y
;; #cd x
;; cd y
;; (pointdir-directory)

;; cf comint-filename-completion in comint-dynamic-complete-functions

(defadvice completion-at-point (around pointdir activate)
  "Default directory based on point."
  (let ((default-directory (pointdir-directory)))
    ;; (message "completion-at-point default-directory %S" default-directory)
    ad-do-it))




;; (defun my-show-cd ()
;;   (interactive)
;;   (message "%S" (pointdir-directory)))
;; (define-key global-map [f2] 'my-show-cd)

;; (defmacro pointdir-with-default-directory (&rest body)
;;   "Evaluate BODY with `default-directory' following any \"cd\" in the buffer.
;; `default-directory' is restored after BODY, except that if BODY
;; changes it then the new value is not overwritten."
;;   `(let ((default-directory (pointdir-directory)))
;;      ,@body))
;; (put 'pointdir-with-default-directory 'lisp-indent-function 0)
;; 
;; ;;;###autoload
;; (defun pointdir--comint-dynamic-complete-filename ()
;;   (interactive)
;;   (require 'comint)
;;   (let ((default-directory (pointdir-directory)))
;;     (comint-dynamic-complete-filename)))
;; 
;; ;;;###autoload
;; (defun pointdir--minibuffer-complete-shell-command ()
;;   (interactive)
;;   (let ((default-directory (pointdir-directory)))
;;     (message "default-directory %S" default-directory)
;;     (if (fboundp 'minibuffer-complete-shell-command)
;;         (minibuffer-complete-shell-command)
;;       (completion-at-point))))
;; 
;; ;;;###autoload
;; (defun pointdir-minibuffer-setup ()
;;   (if (boundp 'minibuffer-local-shell-command-map)
;;       (define-key minibuffer-local-shell-command-map "\t"
;;         'pointdir---minibuffer-complete-shell-command)
;;     (define-key minibuffer-local-map "\t"
;;       'pointdir--comint-dynamic-complete-filename)))
;; 
;; ;; (debug-on-entry 'define-key)
;; ;; (cancel-debug-on-entry 'define-key)
;; 
;; ;; Note no custom-add-option on `minibuffer-setup-hook' since as of emacs23
;; ;; it's only a defvar, not a defcustom.  If it was a defcustom then an
;; ;; option could have been a good way for novice users to enable.
;; ;;
;; ;; ;;;###autoload
;; ;; (custom-add-option 'minibuffer-setup-hook
;; ;;                    'completion-ignored-build-enable-setup)
;; 
;; 
;; ;;-----------------------------------------------------------------------------
;; 
;; (defadvice ffap-file-at-point (around pointdir)
;;   (let ((pointdir--newdir (pointdir-directory)))
;;     (let ((default-directory pointdir--newdir))
;;       ad-do-it)
;;     (if (and ad-return-value
;;              (not (equal pointdir--newdir default-directory)))
;;         (let ((default-directory pointdir--newdir))
;;           (setq ad-return-value (expand-file-name ad-return-value))))))
;; 
;; ;;;###autoload
;; (defun pointdir--ffap-enable ()
;;   (interactive)
;;   (ad-enable-advice 'ffap-file-at-point 'around 'pointdir)
;;   (ad-activate      'ffap-file-at-point))

;; cd /tmp
;; .ICE-unix


;; (defmacro pointdir-with-default-directory (&rest body)
;;   "Evaluate BODY with `default-directory' following any \"cd\" in the buffer.
;; `default-directory' is restored after BODY, except that if BODY
;; changes it then the new value is not overwritten."
;;   `(let ((pointdir-with-default-directory--old default-directory)
;;          (pointdir-with-default-directory--new (pointdir-directory)))
;;      (unwind-protect
;;          (progn
;;            (if pointdir-with-default-directory--new
;;                (setq default-directory
;;                      pointdir-with-default-directory--new))
;;            ,@body)
;;        (if (equal default-directory pointdir-with-default-directory--new)
;;            (setq default-directory
;;                  pointdir-with-default-directory--old)))))

;; ;;-----------------------------------------------------------------------------
;; 
;; (defvar pointdir--advised-functions
;;   '(minibuffer-complete-shell-command
;;     ;; comint-dynamic-complete-filename
;;     ))
;; 
;; (defadvice minibuffer-complete-shell-command (around pointdir)
;;   (pointdir-with-default-directory
;;     ad-do-it))
;; 
;; ;; (defadvice comint-dynamic-complete-filename (around pointdir)
;; ;;   (pointdir-with-default-directory
;; ;;     ad-do-it))
;; 
;; ;;;###autoload
;; (define-minor-mode pointdir-mode
;;   "Recognise \"cd\" commands in the minibuffer for completion.
;; When entering a filename in the minibuffer certain hairy setups
;; are applied to exclude built files for tab completion, for quick
;; access to source files.
;; 
;; --------
;; The pointdir home page is
;; URL `http://user42.tuxfamily.org/pointdir/index.html'"
;; 
;;   :type 'boolean
;;   (remove-hook 'minibuffer-setup-hook
;;                'pointdir-enable-setup)
;;   (let ((action (if pointdir-mode 'ad-enable-advice 'ad-disable-advice)))
;;     (dolist (func pointdir--advised-functions)
;;       (funcall action func 'around 'pointdir)
;;       (ad-activate func))))
;; 
;; ;;;###autoload
;; (defun pointdir-enable-setup ()
;;   "Enable `pointdir-mode' initially.
;; This function is designed for use from `minibuffer-setup-hook'.
;; 
;; It enables `pointdir-mode' and then removes
;; itself from `minibuffer-setup-hook'.  The effect is to load and
;; enable when you first use the minibuffer.  Waiting for the first
;; minibuffer rather than from .emacs can make Emacs startup a
;; little faster.
;; 
;; `pointdir-mode' always removes this from
;; `minibuffer-setup-hook' too, on the basis that if you call it
;; explicitly then that should trump any \"enable-initially\"
;; setup."
;;   (pointdir-mode 1))
;; 
;; (defun pointdir-unload-function ()
;;   (dolist (func pointdir--advised-functions)
;;     (when (ad-find-advice func 'around 'pointdir)
;;       (ad-remove-advice   func 'around 'pointdir)
;;       (ad-activate        func)))
;;   nil) ;; and do normal unload-feature actions too

(provide 'pointdir)

;;; pointdir.el ends here





